//-----------------------------------------
// LScript Generic template
//
// Copyright 2008 Michael Green
// mike@mikegreen.name
//

myversion=1.3;
// v1.3
// Added the ability to fade phonemes if the preceeding and following phonemes are close to them, much like the human mouth skips shapes.
// v1.2
// papagayo CAN work with differing fps, but you have to press enter after changing the fps setting.
// v1.1
// Added help text
// Still to do: 
// More than 1 voice (need sample)
// End capping

@version 2.2
@warnings
@script generic

// global values go here
gset;
st_filen=1; // pgo filename
st_inter=2; // Interpolation type
st_prefi=3; // Morph group prefix
st_streg=4; // Strength of morphs
st_antic=5; //anticipation time (secs)
st_pause=6; // Pause before putting in 'rest' morph    (secs)
st_useco=7; // Use cut off
st_fifps=8; // fps used to make .dat file
st_fadef=9; // fade close frames
gfps=24;

generic
{
	// default
    gset[st_filen]="Sounds/*.dat";
    gset[st_inter]=2;
    gset[st_prefi]="pap";
    gset[st_streg]=1.0;
    gset[st_antic]=2/Scene().fps;
    gset[st_pause]=.5;
    gset[st_useco]=0;
    gset[st_fifps]=25;
    gset[st_fadef]=4;

	for(i=1;i<=gset.size();i++)
	{
		gset[i]=recall("MG_Pap"+i.asStr(),gset[i]);
	}

    reqbegin("MG Papagayo Importer V"+myversion);

    c[st_filen] = ctlfilename("Papagayo file",gset[st_filen],30,1);
    c[st_fifps] = ctlnumber("Papagayo file fps",gset[st_fifps]);
    c[st_inter] = ctlchoice("Interpolation",gset[st_inter],@"Hard","Linear","Smooth"@);
    c[st_prefi] = ctlstring("Phoneme Morph Group Prefix",gset[st_prefi],100);
	c[st_streg] = ctlpercent("Phoneme Strength",gset[st_streg]);
	c[st_antic] = ctlnumber("Anticipation (secs)",gset[st_antic]);
	c[st_pause] = ctlnumber("Phoneme cutoff (secs)",gset[st_pause]);
	c[st_useco] = ctlcheckbox("Use Cut Off",gset[st_useco]);
	c[st_fadef] = ctlnumber("Fade phonemes closers than (in frames)",gset[st_fadef]);
	t = ctltext("Usage","Name your phoneme morphs with",
					"a prefix and then the Papagayo",
					"phoneme name. For example Pap.AI,",
					"Pap.MBP, Pap.rest etc. Anticipation",
					"shifts the morphs back in time as",
					"we tend to form the mouth shapes",
					"just before making the sound");
    return if !reqpost();

    // get requester control values here
	for(i=1;i<=c.size();i++)
	{
		gset[i]=getvalue(c[i]);
		store("MG_Pap"+i.asStr(),gset[i]);
	}
	
	if(gset[st_prefi]=="")
		gset[st_prefi]="MorphGroup";


    reqend();
    Main();
}

Main
{
	mylist=ReadFile(gset[st_filen]);
	mykeys=FilterKeys(mylist);
	mysel=Scene().getSelect();
	if(!mysel || mysel[1].genus!=MESH)
		error("Please select a mesh");
	MakeAnim(mykeys[1],mysel[1]);
}

ReadFile:pfilename
{
	myfile=File(pfilename,"r");
	if(!myfile)
		error("Please select a .dat file");
	i=0;
	while(!myfile.eof())
	{
		myline=myfile.read();
		myparse=ParseLine(myline);
		i++;
		mydata[i]=myparse;
	}
	return(mydata);
}

FilterKeys:plist
{
	mykeys[1][1]=nil;
	index=0;
	for(i=1;i<=plist.size();i++)
	{
		myline=plist[i];
		if(myline.size()==2)// &&myline[1][1].isDigit())
		{
			index++;
			mykeys[1][index][1]=myline[1].asNum();// time is in frames
			mykeys[1][index][2]=myline[2];
  		}
	}
	return(mykeys);
}

MakeAnim:plist,pobj
{
	myres=AddMorphMixer(pobj);
	mychans=GetChannels(pobj,gset[st_prefi]);
	ClearChannels(mychans);
	if(gset[st_inter]==1)
		mytype=CHAN_STEPPED;
	else if(gset[st_inter]==2)
		mytype=CHAN_LINEAR;
	else
		mytype=CHAN_TCB;
	MakeKeys(plist,mychans,mytype,gset[st_fadef]);
}

AddMorphMixer:pobj
{
	myserver=pobj.server(SERVER_DISPLACEMENT_H);
	for(i=1;i<=myserver.size();i++)
	{
		if(myserver[i]=="LW_MorphMixer")
			return(i);
	}
	SelectItem(pobj.id);
	ApplyServer(SERVER_DISPLACEMENT_H,"LW_MorphMixer");
	return(0);
}

GetChannels:pobj,pprefix
{
	mychans[1]=nil;

	mypargroup=ChannelGroup(pobj.name);
	mychangroup=ChannelGroup(mypargroup);
	while(mychangroup)
	{
		if(mychangroup.name==pprefix)
			break;
		mychangroup=ChannelGroup(mypargroup,mychangroup);
	}
	if(!mychangroup)
		error("Phoneme Morph Group "+ pprefix +" not found");

	mychan=mychangroup.firstChannel();
	i=0;
	while(mychan)
	{
			i++;
			mychans[i]=mychan;
		mychan=mychangroup.nextChannel();
	}
	return(mychans);
}

ClearChannels:pchans
{
	for(i=1;i<=pchans.size();i++)
	{
		while(pchans[i].keyCount>=1)
		{
			mykey=pchans[i].keys[1];
			pchans[i].deleteKey(mykey);
		}
		pchans[i].createKey(0,0);
	}
}

MakeKeys:plist,pchans,pinter,pscale=1.0
{
//debug();
	fps=gset[st_fifps];  // you must remember to press enter in Papagayo after changing fps for it to have any effect!!!
	for(i=1;i<=plist.size();i++)
	{
        mytime=(plist[i][1]/fps - gset[st_antic]) ;  //key frame time - anticipation time
        mymorph=plist[i][2];                       // phoneme name
        if( gset[st_useco] && i>1 && mytime > gset[st_pause])
		{
			pausepos= plist[i - 1][1]/fps + gset[st_pause];
  			if(pausepos< plist[i - 1][1]/fps)   // if two consecutive phonemes are greater than 'rest' apart
				MakeKey("rest",pchans,pinter,pausepos,);
 		}
 		myscale=1.0;
 		if(i>1 && i<plist.size() && pscale>1)
 			myscale=min(1.0,(plist[i+1][1] - plist[i- 1][1])/pscale/2);


		MakeKey(mymorph,pchans,pinter,mytime,myscale);
	}
}

MakeKey:pmorph,pchans,pinter,ptime,pscale=1.0
{
	for(j=1;j<=pchans.size();j++)
	{
		if(pchans[j].name==pmorph)
			mykey=pchans[j].createKey(ptime,gset[st_streg]*pscale);
		else
			mykey=pchans[j].createKey(ptime,0);
		pchans[j].setKeyCurve(mykey,pinter);
		if(pinter==CHAN_TCB)
			pchans[j].setKeyTension(mykey,1);
	}
}



ParseLine:pstring
{
	if(pstring==nil)
	{
		mystring[1]=nil;
		return(mystring);
	}
	mysize=pstring.size();
	mystring=pstring;
	i=1;
//	while(i<=mysize && pstring[i].isSpace())
//	{
//    	i++;
//	}
//	mystring=strright(pstring,(mysize - i + 1));
	mystring=parse(" ",mystring);
	return(mystring);
}
